from argparse import ArgumentParser

from satoricli.api import client
from satoricli.cli.utils import (
    BootstrapTable,
    autoformat,
    autotable,
    console,
    group_table,
)

from .base import BaseCommand
from ..utils import get_offset


class MonitorsCommand(BaseCommand):
    name = "monitors"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument(
            "--deleted", action="store_true", help="Show deleted monitors"
        )
        parser.add_argument(
            "--pending", action="store_true", help="Show pending actions"
        )
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)

    def __call__(self, deleted: bool, pending: bool, page: int, limit: int, **kwargs):
        offset = get_offset(page, limit)
        monitors = client.get(
            "/monitors",
            params={
                "deleted": deleted,
                "limit": limit,
                "offset": offset,
            },
        ).json()

        if not kwargs["json"]:
            # Only get pending monitors when is not a json output and on first page
            if page == 1 and pending:
                pending_monitors = client.get("/monitors/pending").json()
                if pending_monitors["rows"]:
                    console.rule("[b red]Pending actions", style="red")
                    autotable(pending_monitors["rows"], "b red", widths=(50, 50))
            console.rule("[b blue]Monitors", style="blue")
            group_table(
                BootstrapTable(**monitors), "team", "Private", page, limit, widths=(16,)
            )
        else:
            autoformat(monitors, jsonfmt=kwargs["json"], list_separator="-" * 48)



from argparse import ArgumentParser
from datetime import date

date_args = ArgumentParser(add_help=False)
date_args.add_argument(
    "--from",
    type=date.fromisoformat,
    help="date in ISO format",
    metavar="DATE",
    dest="from_date",
)
date_args.add_argument(
    "--to",
    type=date.fromisoformat,
    help="date in ISO format",
    metavar="DATE",
    dest="to_date",
)

json_arg = ArgumentParser(add_help=False)
json_arg.add_argument("-j", "--json", action="store_true", help="JSON output")

profile_arg = ArgumentParser(add_help=False)
profile_arg.add_argument("-P", "--profile", default="default")



from argparse import ArgumentParser

from satoricli.api import client

from ..utils import (
    BootstrapTable,
    autoformat,
    autotable,
    console,
    group_table,
    get_offset,
)
from .base import BaseCommand


class ReposCommand(BaseCommand):
    name = "repos"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)
        parser.add_argument(
            "--pending", action="store_true", help="Show pending actions"
        )

    def __call__(self, page: int, limit: int, pending:bool, **kwargs):
        offset = get_offset(page, limit)
        repos = client.get("/repos", params={"offset": offset, "limit": limit}).json()

        if not kwargs["json"]:
            # Only get pending repos when is not a json output and on first page
            if page == 1 and pending:
                pending_repos = client.get("/repos/pending").json()
                if pending_repos["rows"]:
                    console.rule("[b red]Pending actions", style="red")
                    autotable(pending_repos["rows"], "bold red", widths=(50, 50))
            # Group repos by team name
            group_table(BootstrapTable(**repos), "team", "Private", page, limit)
        else:
            autoformat(repos, jsonfmt=kwargs["json"], list_separator="-" * 48)



import json
import os
import platform
from argparse import ArgumentParser
from base64 import b64decode, b64encode
from dataclasses import asdict
from pathlib import Path
from tempfile import SpooledTemporaryFile
from typing import Optional

import httpx
import yaml
from satori_runner import run
from satoricli.api import client
from satoricli.bundler import make_bundle
from satoricli.cli.commands.report import ReportCommand
from satoricli.validations import validate_parameters

from ..utils import (
    console,
    error_console,
    missing_ymls,
    print_output,
    print_summary,
    validate_config,
)
from .base import BaseCommand


def new_local_run(
    bundle=None, playbook_uri: Optional[str] = None, secrets: Optional[dict] = None
) -> dict:
    return client.post(
        "/runs/local",
        data={
            "secrets": json.dumps(secrets) if secrets else None,
            "playbook_uri": playbook_uri,
        },
        files={"bundle": bundle} if bundle else {"": ""},
    ).json()


class BytesEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, bytes):
            return b64encode(o).decode()
        return super().default(o)


class LocalCommand(BaseCommand):
    name = "local"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("target", metavar="TARGET")
        parser.add_argument(
            "-p",
            "--playbook",
            help="if TARGET is a directory this playbook will be used",
        )
        parser.add_argument("-d", "--data", type=json.loads)
        parser.add_argument("--report", action="store_true")
        parser.add_argument("--output", action="store_true")
        parser.add_argument("--summary", action="store_true")

    def __call__(
        self,
        target: str,
        data: Optional[dict],
        playbook: Optional[str],
        report: bool,
        output: bool,
        summary: bool,
        **kwargs,
    ):
        if data and not validate_parameters(data):
            raise ValueError("Malformed parameters")

        workdir = Path(target) if os.path.isdir(target) else Path()

        if (playbook and "://" in playbook) or "://" in target:
            local_run = new_local_run(playbook_uri=playbook or target, secrets=data)
        elif (playbook and os.path.isfile(playbook)) or os.path.isfile(target):
            path = Path(playbook or target)

            if not validate_config(path, set(data.keys()) if data else set()):
                return 1

            bundle = make_bundle(path, path.parent)
            local_run = new_local_run(bundle, secrets=data)
        elif os.path.isdir(target):
            playbook_path = workdir / ".satori.yml"
            config = yaml.safe_load(playbook_path.read_bytes())

            if not validate_config(playbook_path, set(data.keys()) if data else set()):
                return 1

            bundle = make_bundle(playbook_path, playbook_path.parent)

            if missing_ymls(config, workdir):
                error_console.print(
                    "[warning]WARNING:[/] There are some .satori.yml outside the root "
                    "folder that have not been imported."
                )

            local_run = new_local_run(bundle, secrets=data)
        else:
            error_console.print("ERROR: Invalid args")
            return 1

        report_id = local_run["report_id"]

        if not kwargs["json"]:
            console.print("Report ID:", report_id)
            console.print(f"Report: https://satori.ci/report_details/?n={report_id}")
        elif kwargs["json"]:
            console.print_json(data={"report_id": report_id})

        with (
            httpx.stream("GET", local_run["recipe"]) as s,
            SpooledTemporaryFile(4096) as results,
        ):
            os.chdir(workdir)

            for line in s.iter_lines():
                message: dict = json.loads(line)
                command = message["command"]

                windows_host = platform.system() == "Windows"

                if isinstance(command, str):
                    dec = b64decode(command)
                    args = (dec.decode(errors="ignore")) if windows_host else dec
                elif isinstance(command, list):
                    args = [
                        b64decode(arg).decode(errors="ignore")
                        if windows_host
                        else b64decode(arg)
                        for arg in command
                    ]

                out = run(args, message.get("settings", {}).get("setCommandTimeout"))
                message["output"] = asdict(out)
                results.write(f"{json.dumps(message, cls=BytesEncoder)}\n".encode())

            results.seek(0)
            client.put(f"/runs/local/{report_id}", files={"results": results})

        if summary:
            print_summary(report_id, kwargs["json"])

        if report:
            ReportCommand.print_report_asrt(report_id, kwargs["json"])

        if output:
            print_output(report_id, kwargs["json"])
